#ident	"@(#)kern_clock.c 1.1 92/07/30 SMI; from UCB 7.1 6/5/86"

/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#include <sys/param.h>
#include <sys/systm.h>
#include <sys/dk.h>
#include <sys/callout.h>
#include <sys/user.h>
#include <sys/kernel.h>
#include <sys/proc.h>
#include <sys/vm.h>
#include <sys/trace.h>
#include <sys/timex.h>

#include <machine/reg.h>
#include <machine/psl.h>
#include <machine/cpu.h>
#include <machine/param.h>
#ifdef sun4
#include <machine/mmu.h>	/* to get MDEVBASE used in clock.h */
#endif
#include <machine/clock.h>

#include <vm/hat.h>
#include <vm/as.h>
#include <vm/rm.h>

#ifdef vax
#include <vax/mtpr.h>
#endif

#ifdef GPROF
#include <sys/gprof.h>
#endif

#ifdef NOPERCPU
#undef MULTIPROCESSOR
#endif

#ifdef MULTIPROCESSOR
#include "percpu.h"

extern	int	nmod;
#endif

/*
 * Clock handling routines.
 *
 * This code is written to operate with two timers which run
 * independently of each other. The main clock, running at hz
 * times per second, is used to do scheduling and timeout calculations.
 * The second timer does resource utilization estimation statistically
 * based on the state of the machine phz times a second. Both functions
 * can be performed by a single clock (ie hz == phz), however the
 * statistics will be much more prone to errors. Ideally a machine
 * would have separate clocks measuring time spent in user state, system
 * state, interrupt state, and idle state. These clocks would allow a non-
 * approximate measure of resource utilization.
 */

/*
 * TODO:
 *	time of day, system/user timing, timeouts, profiling on separate timers
 *	allocate more timeout table slots when table overflows.
 */
#ifdef notdef
/*
 * Bump a timeval by a small number of usec's.
 */
bumptime(tp, usec)
	register struct timeval *tp;
	int usec;
{

	tp->tv_usec += usec;
	if (tp->tv_usec >= 1000000) {
		tp->tv_usec -= 1000000;
		tp->tv_sec++;
	}
}
#endif notdef
#define	BUMPTIME(t, usec) { \
	register struct timeval *tp = (t); \
	\
	tp->tv_usec += (usec); \
	if (tp->tv_usec >= 1000000) { \
		tp->tv_usec -= 1000000; \
		tp->tv_sec++; \
	} \
}

#ifdef		TRACE
static		int	traceint = 0;
#define		TRACEINT	50
#endif		TRACE

extern struct user *uunix;

/*
 * Phase/frequency-lock loop (PLL/FLL) definitions
 *
 * The following variables are read and set by the ntp_adjtime() system
 * call.
 *
 * time_state shows the state of the system clock, with values defined
 * in the timex.h header file.
 *
 * time_status shows the status of the system clock, with bits defined
 * in the timex.h header file.
 *
 * time_offset is used by the PLL/FLL to adjust the system time in small
 * increments.
 *
 * time_constant determines the bandwidth or "stiffness" of the PLL.
 *
 * time_tolerance determines maximum frequency error or tolerance of the
 * CPU clock oscillator and is a property of the architecture; however,
 * in principle it could change as result of the presence of external
 * discipline signals, for instance.
 *
 * time_precision is usually equal to the kernel tick variable; however,
 * in cases where a precision clock counter or external clock is
 * available, the resolution can be much less than this and depend on
 * whether the external clock is working or not.
 *
 * time_maxerror is initialized by a ntp_adjtime() call and increased by
 * the kernel once each second to reflect the maximum error bound
 * growth.
 *
 * time_esterror is set and read by the ntp_adjtime() call, but
 * otherwise not used by the kernel.
 */
int time_state = TIME_OK;	/* clock state */
int time_status = STA_UNSYNC;	/* clock status bits */
long time_offset = 0;		/* time offset (us) */
long time_constant = 0;		/* pll time constant */
long time_tolerance = MAXFREQ;	/* frequency tolerance (scaled ppm) */
long time_precision = 1;	/* clock precision (us) */
long time_maxerror = MAXPHASE;	/* maximum error (us) */
long time_esterror = MAXPHASE;	/* estimated error (us) */

/*
 * The following variables establish the state of the PLL/FLL and the
 * residual time and frequency offset of the local clock. The scale
 * factors are defined in the timex.h header file.
 *
 * time_phase and time_freq are the phase increment and the frequency
 * increment, respectively, of the kernel time variable.
 *
 * time_freq is set via ntp_adjtime() from a value stored in a file when
 * the synchronization daemon is first started. Its value is retrieved
 * via ntp_adjtime() and written to the file about once per hour by the
 * daemon.
 *
 * time_adj is the adjustment added to the value of tick at each timer
 * interrupt and is recomputed from time_phase and time_freq at each
 * seconds rollover.
 *
 * time_reftime is the second's portion of the system time at the last
 * call to ntp_adjtime(). It is used to adjust the time_freq variable
 * and to increase the time_maxerror as the time since last update
 * increases.
 */
long time_phase = 0;		/* phase offset (scaled us) */
long time_freq = 0;		/* frequency offset (scaled ppm) */
long time_adj = 0;		/* tick adjust (scaled 1 / hz) */
long time_reftime = 0;		/* time at last adjustment (s) */

#ifdef PPS_SYNC
/*
 * The following variables are used only if the kernel PPS discipline
 * code is configured (PPS_SYNC). The scale factors are defined in the
 * timex.h header file.
 *
 * pps_time contains the time at each calibration interval, as read by
 * microtime(). pps_count counts the seconds of the calibration
 * interval, the duration of which is nominally pps_shift in powers of
 * two.
 *
 * pps_offset is the time offset produced by the time median filter
 * pps_tf[], while pps_jitter is the dispersion (jitter) measured by
 * this filter.
 *
 * pps_freq is the frequency offset produced by the frequency median
 * filter pps_ff[], while pps_stabil is the dispersion (wander) measured
 * by this filter.
 *
 * pps_usec is latched from a high resolution counter or external clock
 * at pps_time. Here we want the hardware counter contents only, not the
 * contents plus the time_tv.usec as usual.
 *
 * pps_valid counts the number of seconds since the last PPS update. It
 * is used as a watchdog timer to disable the PPS discipline should the
 * PPS signal be lost.
 *
 * pps_glitch counts the number of seconds since the beginning of an
 * offset burst more than tick/2 from current nominal offset. It is used
 * mainly to suppress error bursts due to priority conflicts between the
 * PPS interrupt and timer interrupt.
 *
 * pps_intcnt counts the calibration intervals for use in the interval-
 * adaptation algorithm. It's just too complicated for words.
 */
struct timeval pps_time;	/* kernel time at last interval */
long pps_tf[] = {0, 0, 0};	/* pps time offset median filter (us) */
long pps_offset = 0;		/* pps time offset (us) */
long pps_jitter = MAXTIME;	/* time dispersion (jitter) (us) */
long pps_ff[] = {0, 0, 0};	/* pps frequency offset median filter */
long pps_freq = 0;		/* frequency offset (scaled ppm) */
long pps_stabil = MAXFREQ;	/* frequency dispersion (scaled ppm) */
long pps_usec = 0;		/* microsec counter at last interval */
long pps_valid = PPS_VALID;	/* pps signal watchdog counter */
int pps_glitch = 0;		/* pps signal glitch counter */
int pps_count = 0;		/* calibration interval counter (s) */
int pps_shift = PPS_SHIFT;	/* interval duration (s) (shift) */
int pps_intcnt = 0;		/* intervals at current duration */

/*
 * PPS signal quality monitors
 *
 * pps_jitcnt counts the seconds that have been discarded because the
 * jitter measured by the time median filter exceeds the limit MAXTIME
 * (100 us).
 *
 * pps_calcnt counts the frequency calibration intervals, which are
 * variable from 4 s to 256 s.
 *
 * pps_errcnt counts the calibration intervals which have been discarded
 * because the wander exceeds the limit MAXFREQ (100 ppm) or where the
 * calibration interval jitter exceeds two ticks.
 *
 * pps_stbcnt counts the calibration intervals that have been discarded
 * because the frequency wander exceeds the limit MAXFREQ / 4 (25 us).
 */
long pps_jitcnt = 0;		/* jitter limit exceeded */
long pps_calcnt = 0;		/* calibration intervals */
long pps_errcnt = 0;		/* calibration errors */
long pps_stbcnt = 0;		/* stability limit exceeded */
#endif /* PPS_SYNC */

#ifdef EXT_CLOCK
/*
 * External clock definitions
 *
 * The following definitions and declarations are used only if an
 * external clock is configured on the system.
 */
#define CLOCK_INTERVAL 30	/* CPU clock update interval (s) */

/*
 * The clock_count variable is set to CLOCK_INTERVAL at each PPS
 * interrupt and decremented once each second.
 */
int clock_count = 0;		/* CPU clock counter */

#ifdef HIGHBALL
/*
 * The clock_offset and clock_cpu variables are used by the HIGHBALL
 * interface. The clock_offset variable defines the offset between
 * system time and the HIGBALL counters. The clock_cpu variable contains
 * the offset between the system clock and the HIGHBALL clock for use in
 * disciplining the kernel time variable.
 */
extern struct timeval clock_offset; /* Highball clock offset */
long clock_cpu = 0;		/* CPU clock adjust */
#endif /* HIGHBALL */
#endif /* EXT_CLOCK */

struct timeval delta;		/* debug */

/*
 * The hz hardware interval timer.
 * We update the events relating to real time.
 * If this timer is also being used to gather statistics,
 * we run through the statistics gathering routine as well.
 */
#ifdef MEASURE_KERNELSTACK
/*
 * Instrumented to measure the stack pointer
 *
 */
int minkernstack = 0;
int minintrstack = 0;
int print_at_all = 0;
int print_minkernstack = 0;
int print_minintrstack = 0;

#include <machine/pte.h>

hardclock(pc, ps, sp)
	caddr_t pc;
	int ps;
	int sp;	/* Yes, should be a caddr_t, but making it an int is easier */
#else	MEASURE_KERNELSTACK
hardclock(pc, ps)
	caddr_t pc;
	int ps;
#endif	MEASURE_KERNELSTACK
{
	register struct callout *p1;
#ifndef MULTIPROCESSOR
	register struct proc *p;
	register int s;
#endif
	int needsoft = 0;
	extern int tickdelta;
	extern long timedelta;
	int time_update;
	long ltemp;
#ifdef EXT_CLOCK
	struct timeval clock_ext;
#endif /* EXT_CLOCK */

	/*
	 * doresettodr values:
	 *   0 - 	no set pending
	 *   1 -	set when wanting to be sync'd
	 *   other -	set chip when time.tv_sec greater
	 *		than or equal to this value
	 */
	extern int doresettodr;

	/*
	 * We increment the tick early in this routine to minimize the
	 * probability that the PPS interrupt routine calls microtime()
	 * before the tick is advanced. While this is a dirty rotten
	 * crock, it avoids messing with interrupt linkages. In any
	 * case, the PPS code has provisions to avoid jitter due this
	 * and other crocks.
	 */
	time.tv_usec += tick;
	time_update = 0;
	if (timedelta == 0) {
		if (doresettodr == 1) {
			/*
			 * Oh dear- somebody zeroed timedelta out from
			 * underneath an adjustment that was in progress.
			 * Better zero doresettodr to be sure...
			 */
			doresettodr = 0;
		}
	} else {
		if (timedelta < 0) {
			time_update -= tickdelta;
			timedelta += tickdelta;
		} else {
			time_update += tickdelta;
			timedelta -= tickdelta;
		}

		if (-tickdelta < timedelta && timedelta < tickdelta) {
			timedelta = 0;
			if (doresettodr == 1) {
				doresettodr = time.tv_sec + 1;
			}
		}
	}

	/*
	 * Beginning of precision-kernel code fragment
	 *
	 * Compute the phase adjustment. If the low-order bits
	 * (time_phase) of the update overflow, bump the high-order bits
	 * (time_update).
	 */
	time_phase += time_adj;
	if (time_phase <= -FINEUSEC) {
		ltemp = -time_phase >> SHIFT_SCALE;
		time_phase += ltemp << SHIFT_SCALE;
		time_update -= ltemp;
	}
	else if (time_phase >= FINEUSEC) {
		ltemp = time_phase >> SHIFT_SCALE;
		time_phase -= ltemp << SHIFT_SCALE;
		time_update += ltemp;
	}

#ifdef HIGHBALL
	/*
	 * If the HIGHBALL board is installed, we need to adjust the
	 * external clock offset in order to close the hardware feedback
	 * loop. This will adjust the external clock phase and frequency
	 * in small amounts. The additional phase noise and frequency
	 * wander this causes should be minimal. We also need to
	 * discipline the kernel time variable, since the PLL is used to
	 * discipline the external clock. If the Highball board is not
	 * present, we discipline kernel time with the PLL as usual. We
	 * assume that the external clock phase adjustment (time_update)
	 * and kernel phase adjustment (clock_cpu) are less than the
	 * value of tick.
	 */
	clock_offset.tv_usec += time_update;
	if (clock_offset.tv_usec >= 1000000) {
		clock_offset.tv_sec++;
		clock_offset.tv_usec -= 1000000;
	}
	if (clock_offset.tv_usec < 0) {
		clock_offset.tv_sec--;
		clock_offset.tv_usec += 1000000;
	}
	time.tv_usec += clock_cpu;
	clock_cpu = 0;
#else
	time.tv_usec += time_update;
#endif /* HIGHBALL */

	/*
	 * Beginning of precision-kernel code fragment
	 *
	 * On rollover of the second the phase adjustment to be used for
	 * the next second is calculated. Also, the maximum error is
	 * increased by the tolerance. If the PPS frequency discipline
	 * code is present, the phase is increased to compensate for the
	 * CPU clock oscillator frequency error.
	 *
 	 * On a 32-bit machine and given parameters in the timex.h
	 * header file, the maximum phase adjustment is +-512 ms and
	 * maximum frequency offset is (a tad less than) +-512 ppm. On a
	 * 64-bit machine, you shouldn't need to ask.
	 */
	if (time.tv_usec >= 1000000) {
		time.tv_usec -= 1000000;
		time.tv_sec++;
		time_maxerror += time_tolerance >> SHIFT_USEC;

		/*
		 * Leap second processing. If in leap-insert state at
		 * the end of the day, the system clock is set back one
		 * second; if in leap-delete state, the system clock is
		 * set ahead one second. The microtime() routine or
		 * external clock driver will insure that reported time
		 * is always monotonic. The ugly divides should be
		 * replaced.
		 */
		switch (time_state) {

			case TIME_OK:
			if (time_status & STA_INS)
				time_state = TIME_INS;
			else if (time_status & STA_DEL)
				time_state = TIME_DEL;
			break;

			case TIME_INS:
			if (time.tv_sec % 86400 == 0) {
				time.tv_sec--;
				time_state = TIME_OOP;
			}
			break;

			case TIME_DEL:
			if ((time.tv_sec + 1) % 86400 == 0) {
				time.tv_sec++;
				time_state = TIME_WAIT;
			}
			break;

			case TIME_OOP:
			time_state = TIME_WAIT;
			break;

			case TIME_WAIT:
			if (!(time_status & (STA_INS | STA_DEL)))
				time_state = TIME_OK;
		}

		/*
		 * Compute the phase adjustment for the next second. In
		 * PLL mode, the offset is reduced by a fixed factor
		 * times the time constant. In FLL mode the offset is
		 * used directly. In either mode, the maximum phase
		 * adjustment for each second is clamped so as to spread
		 * the adjustment over not more than the number of
		 * seconds between updates.
		 */
		if (time_offset < 0) {
			ltemp = -time_offset;
			if (!(time_status & STA_FLL))
				ltemp >>= SHIFT_KG + time_constant;
			if (ltemp > (MAXPHASE / MINSEC) << SHIFT_UPDATE)
				ltemp = (MAXPHASE / MINSEC) <<
				    SHIFT_UPDATE;
			time_offset += ltemp;
			time_adj = -ltemp << (SHIFT_SCALE - SHIFT_HZ -
			    SHIFT_UPDATE);
		} else {
			ltemp = time_offset;
			if (!(time_status & STA_FLL))
				ltemp >>= SHIFT_KG + time_constant;
			if (ltemp > (MAXPHASE / MINSEC) << SHIFT_UPDATE)
				ltemp = (MAXPHASE / MINSEC) <<
				    SHIFT_UPDATE;
			time_offset -= ltemp;
			time_adj = ltemp << (SHIFT_SCALE - SHIFT_HZ -
			    SHIFT_UPDATE);
		}

		/*
		 * Compute the frequency estimate and additional phase
		 * adjustment due to frequency error for the next
		 * second. When the PPS signal is engaged, gnaw on the
		 * watchdog counter and update the frequency computed by
		 * the pll and the PPS signal.
		 */
#ifdef PPS_SYNC
		pps_valid++;
		if (pps_valid == PPS_VALID) {
			pps_jitter = MAXTIME;
			pps_stabil = MAXFREQ;
			time_status &= ~(STA_PPSSIGNAL | STA_PPSJITTER |
			    STA_PPSWANDER | STA_PPSERROR);
		}
		ltemp = time_freq + pps_freq;
#else
		ltemp = time_freq;
#endif /* PPS_SYNC */
		if (ltemp < 0)
			time_adj -= -ltemp >>
			    (SHIFT_USEC + SHIFT_HZ - SHIFT_SCALE);
		else
			time_adj += ltemp >>
			    (SHIFT_USEC + SHIFT_HZ - SHIFT_SCALE);

#if SHIFT_HZ == 7
		/*
		 * When the CPU clock oscillator frequency is not a
		 * power of 2 in Hz, the SHIFT_HZ is only an approximate
		 * scale factor. In the SunOS kernel, this results in a
		 * PLL gain factor of 1/1.28 = 0.78 what it should be.
		 * In the following code the overall gain is increased
		 * by a factor of 1.25, which results in a residual
		 * error less than 3 percent.
		 */
		if (hz == 100) {
			if (time_adj < 0)
				time_adj -= -time_adj >> 2;
			else
				time_adj += time_adj >> 2;
		}
#endif /* SHIFT_HZ */

#ifdef EXT_CLOCK
		/*
		 * If an external clock is present, it is necessary to
		 * discipline the kernel time variable anyway, since not
		 * all system components use the microtime() interface.
		 * Here, the time offset between the external clock and
		 * kernel time variable is computed every so often.
		 */
		clock_count++;
		if (clock_count > CLOCK_INTERVAL) {
			clock_count = 0;
			microtime(&clock_ext);
			delta.tv_sec = clock_ext.tv_sec - time.tv_sec;
			delta.tv_usec = clock_ext.tv_usec -
			    time.tv_usec;
			if (delta.tv_usec < 0)
				delta.tv_sec--;
			if (delta.tv_usec >= 500000) {
				delta.tv_usec -= 1000000;
				delta.tv_sec++;
			}
			if (delta.tv_usec < -500000) {
				delta.tv_usec += 1000000;
				delta.tv_sec--;
			}
			if (delta.tv_sec > 0 || (delta.tv_sec == 0 &&
			    delta.tv_usec > MAXPHASE) ||
			    delta.tv_sec < -1 || (delta.tv_sec == -1 &&
			    delta.tv_usec < -MAXPHASE)) {
				time = clock_ext;
				delta.tv_sec = 0;
				delta.tv_usec = 0;
			}
#ifdef HIGHBALL 
			clock_cpu = delta.tv_usec;
#else /* HIGHBALL */
			hardupdate(delta.tv_usec);
#endif /* HIGHBALL */
		}
#endif /* EXT_CLOCK */
	}

	/*
	 * End of precision-kernel code fragment
	 *
	 * The following code was moved here from the beginning of
	 * hardclock() to reduce the latency to update the time
	 * variable.
	 *
	 * Increment the time-of-day, and schedule processing of the
	 * callouts at a very low cpu priority, so we don't keep the
	 * relatively high clock interrupt priority any longer than
	 * necessary.
	 */
#ifdef MEASURE_KERNELSTACK
	/*
	 * Find the lowest stack pointer and print it out
	 */
	if (! USERMODE(ps)) {
		if (sp > (int) Sysmap) {
			/* On kernel stack */
			if (sp < minkernstack) {
				minkernstack = sp;
				if (print_at_all) {
					print_minkernstack = 1;
					needsoft = 1;
				}
			}
		} else {
			/* On interrupt stack */
			if (sp < minintrstack) {
				minintrstack = sp;
				if (print_at_all) {
					print_minintrstack = 1;
					needsoft = 1;
				}
			}
		}
	}
#endif MEASURE_KERNELSTACK

#ifdef	TRACE
	/* Calibrate trace */
	if (++traceint == TRACEINT) {
		traceint = 0;
		trace3(TR_CALIBRATE, 0, time.tv_sec, time.tv_usec);
		trace3(TR_CALIBRATE, 1, time.tv_sec, time.tv_usec);
	}
#endif	TRACE

	/*
	 * Update real-time timeout queue.
	 * At front of queue are some number of events which are ``due''.
	 * The time to these is <= 0 and if negative represents the
	 * number of ticks which have passed since it was supposed to happen.
	 * The rest of the q elements (times > 0) are events yet to happen,
	 * where the time for each is given as a delta from the previous.
	 * Decrementing just the first of these serves to decrement the time
	 * to all events.
	 */
	p1 = calltodo.c_next;
	while (p1) {
		if (--p1->c_time > 0)
			break;
		needsoft = 1;
		if (p1->c_time == 0)
			break;
		p1 = p1->c_next;
	}

#ifndef MULTIPROCESSOR
	/*
	 * Charge the time out based on the mode the cpu is in.
	 * Here again we fudge for the lack of proper interval timers
	 * assuming that the current state has been around at least
	 * one tick.
	 */
	if (USERMODE(ps)) {
		if (u.u_prof.pr_scale)
			needsoft = 1;
		/*
		 * CPU was in user state.  Increment
		 * user time counter, and process process-virtual time
		 * interval timer.
		 */
		BUMPTIME(&u.u_ru.ru_utime, tick);
		if (timerisset(&u.u_timer[ITIMER_VIRTUAL].it_value) &&
		    itimerdecr(&u.u_timer[ITIMER_VIRTUAL], tick) == 0)
			psignal(u.u_procp, SIGVTALRM);
	} else {
		/*
		 * CPU was in system state.
		 */
		if (!noproc) {
			BUMPTIME(&u.u_ru.ru_stime, tick);
		}
	}

	/*
	 * If the cpu is currently scheduled to a process, then
	 * charge it with resource utilization for a tick, updating
	 * statistics which run in (user+system) virtual time,
	 * such as the cpu time limit and profiling timers.
	 * This assumes that the current process has been running
	 * the entire last tick.
	 */
	if (!noproc && !(u.u_procp->p_flag & SWEXIT)) {
		p = u.u_procp;
		if ((u.u_ru.ru_utime.tv_sec+u.u_ru.ru_stime.tv_sec+1) >
		    u.u_rlimit[RLIMIT_CPU].rlim_cur) {
			psignal(p, SIGXCPU);
			if (u.u_rlimit[RLIMIT_CPU].rlim_cur <
			    u.u_rlimit[RLIMIT_CPU].rlim_max)
				u.u_rlimit[RLIMIT_CPU].rlim_cur += 5;
		}
		if (timerisset(&u.u_timer[ITIMER_PROF].it_value) &&
		    itimerdecr(&u.u_timer[ITIMER_PROF], tick) == 0)
			psignal(p, SIGPROF);
		s = p->p_rssize = rm_asrss(p->p_as);
		u.u_ru.ru_idrss += s;
		if (s > u.u_ru.ru_maxrss)
			u.u_ru.ru_maxrss = s;
		/*
		* We adjust the priority of the current process.
		* The priority of a process gets worse as it accumulates
		* CPU time.  The cpu usage estimator (p_cpu) is increased here
		* and the formula for computing priorities (in kern_synch.c)
		* will compute a different value each time the p_cpu increases
		* by 4.  The cpu usage estimator ramps up quite quickly when the
		* process is running (linearly), and decays away exponentially,
		* at a rate which is proportionally slower when the system is
		* busy.  The basic principal is that the system will 90% forget
		* that a process used a lot of CPU time in 5*loadav seconds.
		* [See def. of nrscale in kern_synch.c for how this works.]
		* This causes the system to favor processes which haven't run
		* much recently, and to round-robin among other processes.
		* NOTE:
		* The scheduling alteration is a non sequitur to kernel
		* running lwp's. In fact, if p_pri changes while the
		* process is still on the run queue, a panic from remrq
		* may result
		*/
#ifdef LWP
		if (!runthreads) {
#endif LWP
			p->p_cpticks++;
			if (++p->p_cpu == 0)
				p->p_cpu--;
			if ((p->p_cpu&3) == 0) {
				(void) setpri(p);
				if (p->p_pri >= PUSER)
					p->p_pri = p->p_usrpri;
			}
#ifdef LWP
		}
#endif LWP
	}

#endif /* !MULTIPROCESSOR */

	/*
	 * If the alternate clock has not made itself known then
	 * we must gather the statistics.
	 */
	if (phz == 0)
		gatherstats(pc, ps);

#ifdef	MULTIPROCESSOR
	charge_time(&needsoft);
#endif

	if (doresettodr > 1 && doresettodr <= time.tv_sec) {
		doresettodr = 0;
		resettodr();
	}

	if (needsoft) {
		if (BASEPRI(ps)) {
			/*
			 * Save the overhead of a software interrupt;
			 * it will happen as soon as we return, so do it now.
			 */
			(void) splsoftclock();
			softclock(ps);
		} else {
#ifdef sun
			int softclock();

			/*
			 * Pass softclock only the system mode bit
			 * to avoid the softcall table from overflowing
			 * from the various different calls to softclock.
			 */
			softcall(softclock, (caddr_t)(ps & SR_SMODE));
#endif
#ifdef vax
			setsoftclock();
#endif
		}
	}
}

/*
 * hardupdate() - local clock update
 *
 * This routine is called by ntp_adjtime() to update the local clock
 * phase and frequency. The implementation is of an adaptive-parameter,
 * hybrid phase/frequency-lock loop (PLL/FLL). The routine computes new
 * time and frequency offset estimates for each call. If the kernel PPS
 * discipline code is configured (PPS_SYNC), the PPS signal itself
 * determines the new time offset, instead of the calling argument.
 * Presumably, calls to ntp_adjtime() occur only when the caller
 * believes the local clock is valid within some bound (+-128 ms with
 * NTP). If the caller's time is far different than the PPS time, an
 * argument will ensue, and it's not clear who will lose.
 *
 * For uncompensated quartz crystal oscillatores and nominal update
 * intervals less than 1024 s, operation should be in phase-lock mode
 * (STA_FLL = 0), where the loop is disciplined to phase. For update
 * intervals greater than thiss, operation should be in frequency-lock
 * mode (STA_FLL = 1), where the loop is disciplined to frequency.
 *
 * Note: splclock() is in effect.
 */
hardupdate(offset)
	long offset;
{
	long ltemp, mtemp;

	if (!(time_status & STA_PLL) && !(time_status & STA_PPSTIME))
		return;
	ltemp = offset;
#ifdef PPS_SYNC
	if (time_status & STA_PPSTIME && time_status & STA_PPSSIGNAL)
		ltemp = pps_offset;
#endif /* PPS_SYNC */

	/*
	 * Scale the phase adjustment and clamp to the operating range.
	 */
	if (ltemp > MAXPHASE)
		time_offset = MAXPHASE << SHIFT_UPDATE;
	else if (ltemp < -MAXPHASE)
		time_offset = -(MAXPHASE << SHIFT_UPDATE);
	else
		time_offset = ltemp << SHIFT_UPDATE;

	/*
	 * Select whether the frequency is to be controlled and in which
	 * mode (PLL or FLL). Clamp to the operating range. Ugly
	 * multiply/divide should be replaced someday.
	 */
	if (time_status & STA_FREQHOLD || time_reftime == 0)
		time_reftime = time.tv_sec;
	mtemp = time.tv_sec - time_reftime;
	time_reftime = time.tv_sec;
	if (time_status & STA_FLL) {
		if (mtemp >= MINSEC) {
			ltemp = ((time_offset / mtemp) << (SHIFT_USEC -
			    SHIFT_UPDATE));
			if (ltemp < 0)
				time_freq -= -ltemp >> SHIFT_KH;
			else
				time_freq += ltemp >> SHIFT_KH;
		}
	} else {
		if (mtemp < MAXSEC) {
			ltemp *= mtemp;
			if (ltemp < 0)
				time_freq -= -ltemp >> (time_constant +
				    time_constant + SHIFT_KF -
				    SHIFT_USEC);
			else
				time_freq += ltemp >> (time_constant +
				    time_constant + SHIFT_KF -
				    SHIFT_USEC);
		}
	}
	if (time_freq > time_tolerance)
		time_freq = time_tolerance;
	else if (time_freq < -time_tolerance)
		time_freq = -time_tolerance;
}

#ifdef PPS_SYNC
/*
 * hardpps() - discipline CPU clock oscillator to external PPS signal
 *
 * This routine is called at each PPS interrupt in order to discipline
 * the CPU clock oscillator to the PPS signal. It measures the PPS phase
 * and leaves it in a handy spot for the hardclock() routine. It
 * integrates successive PPS phase differences and calculates the
 * frequency offset. This is used in hardclock() to discipline the CPU
 * clock oscillator so that intrinsic frequency error is cancelled out.
 * The code requires the caller to capture the time and hardware counter
 * value at the on-time PPS signal transition.
 *
 * Note that, on some Unix systems, this routine runs at an interrupt
 * priority level higher than the timer interrupt routine hardclock().
 * Therefore, the variables used are distinct from the hardclock()
 * variables, except for certain exceptions: The PPS frequency pps_freq
 * and phase pps_offset variables are determined by this routine and
 * updated atomically. The time_tolerance variable can be considered a
 * constant, since it is infrequently changed, and then only when the
 * PPS signal is disabled. The watchdog counter pps_valid is updated
 * once per second by hardclock() and is atomically cleared in this
 * routine.
 */
hardpps(tvp, usec)
	struct timeval *tvp;		/* time at PPS */
	long usec;			/* hardware counter at PPS */
{
	long u_usec, v_usec, bigtick;
	long cal_sec, cal_usec;

	/*
	 * An occasional glitch can be produced when the PPS interrupt
	 * occurs in the hardclock() routine before the time variable is
	 * updated. Here the offset is discarded when the difference
	 * between it and the last one is greater than tick/2, but not
	 * if the interval since the first discard exceeds 30 s.
	 */
	time_status |= STA_PPSSIGNAL;
	time_status &= ~(STA_PPSJITTER | STA_PPSWANDER | STA_PPSERROR);
	pps_valid = 0;
	u_usec = -tvp->tv_usec;
	if (u_usec < -500000)
		u_usec += 1000000;
	v_usec = pps_offset - u_usec;
	if (v_usec < 0)
		v_usec = -v_usec;
	if (v_usec > (tick >> 1)) {
		if (pps_glitch > MAXGLITCH) {
			pps_glitch = 0;
			pps_tf[2] = u_usec;
			pps_tf[1] = u_usec;
		} else {
			pps_glitch++;
			u_usec = pps_offset;
		}
	} else
		pps_glitch = 0;

	/*
	 * A three-stage median filter is used to help deglitch the pps
	 * time. The median sample becomes the time offset estimate; the
	 * difference between the other two samples becomes the time
	 * dispersion (jitter) estimate.
	 */
	pps_tf[2] = pps_tf[1];
	pps_tf[1] = pps_tf[0];
	pps_tf[0] = u_usec;
	if (pps_tf[0] > pps_tf[1]) {
		if (pps_tf[1] > pps_tf[2]) {
			pps_offset = pps_tf[1];		/* 0 1 2 */
			v_usec = pps_tf[0] - pps_tf[2];
		} else if (pps_tf[2] > pps_tf[0]) {
			pps_offset = pps_tf[0];		/* 2 0 1 */
			v_usec = pps_tf[2] - pps_tf[1];
		} else {
			pps_offset = pps_tf[2];		/* 0 2 1 */
			v_usec = pps_tf[0] - pps_tf[1];
		}
	} else {
		if (pps_tf[1] < pps_tf[2]) {
			pps_offset = pps_tf[1];		/* 2 1 0 */
			v_usec = pps_tf[2] - pps_tf[0];
		} else  if (pps_tf[2] < pps_tf[0]) {
			pps_offset = pps_tf[0];		/* 1 0 2 */
			v_usec = pps_tf[1] - pps_tf[2];
		} else {
			pps_offset = pps_tf[2];		/* 1 2 0 */
			v_usec = pps_tf[1] - pps_tf[0];
		}
	}
	if (v_usec > MAXTIME)
		pps_jitcnt++;
	v_usec = (v_usec << PPS_AVG) - pps_jitter;
	if (v_usec < 0)
		pps_jitter -= -v_usec >> PPS_AVG;
	else
		pps_jitter += v_usec >> PPS_AVG;
	if (pps_jitter > (MAXTIME >> 1))
		time_status |= STA_PPSJITTER;

	/*
	 * During the calibration interval adjust the starting time when
	 * the tick overflows. At the end of the interval compute the
	 * duration of the interval and the difference of the hardware
	 * counters at the beginning and end of the interval. This code
	 * is deliciously complicated by the fact valid differences may
	 * exceed the value of tick when using long calibration
	 * intervals and small ticks. Note that the counter can be
	 * greater than tick if caught at just the wrong instant, but
	 * the values returned and used here are correct.
	 */
	bigtick = (long)tick << SHIFT_USEC;
	pps_usec -= pps_freq;
	if (pps_usec >= bigtick)
		pps_usec -= bigtick;
	if (pps_usec < 0)
		pps_usec += bigtick;
	pps_time.tv_sec++;
	pps_count++;
	if (pps_count < (1 << pps_shift))
		return;
	pps_count = 0;
	pps_calcnt++;
	u_usec = usec << SHIFT_USEC;
	v_usec = pps_usec - u_usec;
	if (v_usec >= bigtick >> 1)
		v_usec -= bigtick;
	if (v_usec < -(bigtick >> 1))
		v_usec += bigtick;
	if (v_usec < 0)
		v_usec = -(-v_usec >> pps_shift);
	else
		v_usec = v_usec >> pps_shift;
	pps_usec = u_usec;
	cal_sec = tvp->tv_sec;
	cal_usec = tvp->tv_usec;
	cal_sec -= pps_time.tv_sec;
	cal_usec -= pps_time.tv_usec;
	if (cal_usec < 0) {
		cal_usec += 1000000;
		cal_sec--;
	}
	pps_time = *tvp;

	/*
	 * Check for lost interrupts, noise, excessive jitter and
	 * excessive frequency error. The number of timer ticks during
	 * the interval may vary +-1 tick. Add to this a margin of one
	 * tick for the PPS signal jitter and maximum frequency
	 * deviation. If the limits are exceeded, the calibration
	 * interval is reset to the minimum and we start over.
	 */
	u_usec = (long)tick << 1;
	if (!((cal_sec == -1 && cal_usec > (1000000 - u_usec))
	    || (cal_sec == 0 && cal_usec < u_usec))
	    || v_usec > time_tolerance || v_usec < -time_tolerance) {
		pps_errcnt++;
		pps_shift = PPS_SHIFT;
		pps_intcnt = 0;
		time_status |= STA_PPSERROR;
		return;
	}

	/*
	 * A three-stage median filter is used to help deglitch the pps
	 * frequency. The median sample becomes the frequency offset
	 * estimate; the difference between the other two samples
	 * becomes the frequency dispersion (stability) estimate.
	 */
	pps_ff[2] = pps_ff[1];
	pps_ff[1] = pps_ff[0];
	pps_ff[0] = v_usec;
	if (pps_ff[0] > pps_ff[1]) {
		if (pps_ff[1] > pps_ff[2]) {
			u_usec = pps_ff[1];		/* 0 1 2 */
			v_usec = pps_ff[0] - pps_ff[2];
		} else if (pps_ff[2] > pps_ff[0]) {
			u_usec = pps_ff[0];		/* 2 0 1 */
			v_usec = pps_ff[2] - pps_ff[1];
		} else {
			u_usec = pps_ff[2];		/* 0 2 1 */
			v_usec = pps_ff[0] - pps_ff[1];
		}
	} else {
		if (pps_ff[1] < pps_ff[2]) {
			u_usec = pps_ff[1];		/* 2 1 0 */
			v_usec = pps_ff[2] - pps_ff[0];
		} else  if (pps_ff[2] < pps_ff[0]) {
			u_usec = pps_ff[0];		/* 1 0 2 */
			v_usec = pps_ff[1] - pps_ff[2];
		} else {
			u_usec = pps_ff[2];		/* 1 2 0 */
			v_usec = pps_ff[1] - pps_ff[0];
		}
	}

	/*
	 * Here the frequency dispersion (stability) is updated. If it
	 * is less than one-fourth the maximum (MAXFREQ), the frequency
	 * offset is updated as well, but clamped to the tolerance. It
	 * will be processed later by the hardclock() routine.
	 */
	v_usec = (v_usec >> 1) - pps_stabil;
	if (v_usec < 0)
		pps_stabil -= -v_usec >> PPS_AVG;
	else
		pps_stabil += v_usec >> PPS_AVG;
	if (pps_stabil > MAXFREQ >> 2) {
		pps_stbcnt++;
		time_status |= STA_PPSWANDER;
		return;
	}
	if (time_status & STA_PPSFREQ) {
		if (u_usec < 0) {
			pps_freq -= -u_usec >> PPS_AVG;
			if (pps_freq < -time_tolerance)
				pps_freq = -time_tolerance;
			u_usec = -u_usec;
		} else {
			pps_freq += u_usec >> PPS_AVG;
			if (pps_freq > time_tolerance)
				pps_freq = time_tolerance;
		}
	}

	/*
	 * Here the calibration interval is adjusted. If the maximum
	 * time difference is greater than tick / 4, reduce the interval
	 * by half. If this is not the case for four consecutive
	 * intervals, double the interval.
	 */
	if (u_usec << pps_shift > bigtick >> 2) {
		pps_intcnt = 0;
		if (pps_shift > PPS_SHIFT)
			pps_shift--;
	} else if (pps_intcnt >= 4) {
		pps_intcnt = 0;
		if (pps_shift < PPS_SHIFTMAX)
			pps_shift++;
	} else
		pps_intcnt++;
}
#endif /* PPS_SYNC */

#ifdef MULTIPROCESSOR

/*
 * Charge cpu time to all processes associated with a processor.
 * *needsoftp is set to 1 if a softcall is required.  This is set for
 * the calling routine's (hardclock()) benefit.
 */
charge_time(needsoftp)
	int            *needsoftp;
{
	register int    cix, s;
	register struct user *up;
	register struct proc *p;


	/*
	 * For each processor...
	 */

	for (cix = 0; cix < nmod; cix++) {
		if (!PerCPU[cix].cpu_exists ||
		    PerCPU[cix].noproc)
			continue;
		/*
		 * cpu_idle is set by the idle process when it is sure
		 * that nothing is going on.
		 */
		if (PerCPU[cix].cpu_idle)
			continue;
		up = PerCPU[cix].uunix;
		if (up == NULL)
			continue;
		p = up->u_procp;
		if (p == NULL)
			continue;
		if (p == &idleproc)
			continue;
		/*
		 * Charge the time out based on the mode the cpu is in. Here
		 * again we fudge for the lack of proper interval timers
		 * assuming that the current state has been around at least
		 * one tick.
		 *
		 * XXX - of course, Galaxy has proper interval timers; do we
		 * have the rocket-science time to gut all this stuff and make
		 * it all work?
		 */
		if (PerCPU[cix].cpu_supv) {
			BUMPTIME(&up->u_ru.ru_stime, tick);
		} else {
			if (up->u_prof.pr_scale)
				*needsoftp = 1;
			/*
			 * Increment user time counter, and process
			 * process-virtual time interval timer.
			 */
			BUMPTIME(&up->u_ru.ru_utime, tick);
			if ((timerisset(&up->u_timer[ITIMER_VIRTUAL].it_value)) &&
			    (itimerdecr(&up->u_timer[ITIMER_VIRTUAL], tick) == 0))
				psignal(p, SIGVTALRM);
		}

		/*
		 * If the cpu is currently scheduled to a process, then
		 * charge it with resource utilization for a tick, updating
		 * statistics which run in (user+system) virtual time, such
		 * as the cpu time limit and profiling timers. This assumes
		 * that the current process has been running the entire last
		 * tick.
		 */
		if (!(p->p_flag & SWEXIT)) {
			if ((up->u_ru.ru_utime.tv_sec + up->u_ru.ru_stime.tv_sec + 1) >
			    up->u_rlimit[RLIMIT_CPU].rlim_cur) {
				psignal(p, SIGXCPU);
				if (up->u_rlimit[RLIMIT_CPU].rlim_cur <
				    up->u_rlimit[RLIMIT_CPU].rlim_max)
					up->u_rlimit[RLIMIT_CPU].rlim_cur += 5;
			}
			if (timerisset(&up->u_timer[ITIMER_PROF].it_value) &&
			    itimerdecr(&up->u_timer[ITIMER_PROF], tick) == 0)
				psignal(p, SIGPROF);
			s = p->p_rssize = rm_asrss(p->p_as);
			up->u_ru.ru_idrss += s;
			if (s > up->u_ru.ru_maxrss)
				up->u_ru.ru_maxrss = s;
			/*
			 * We adjust the priority of the current process. The
			 * priority of a process gets worse as it accumulates
			 * CPU time.  The cpu usage estimator (p_cpu) is
			 * increased here and the formula for computing
			 * priorities (in kern_synch.c) will compute a
			 * different value each time the p_cpu increases by
			 * 4.  The cpu usage estimator ramps up quite quickly
			 * when the process is running (linearly), and decays
			 * away exponentially, at a rate which is
			 * proportionally slower when the system is busy.
			 * The basic principal is that the system will 90%
			 * forget that a process used a lot of CPU time in
			 * 5*loadav seconds. [See def. of nrscale in
			 * kern_synch.c for how this works.] This causes the
			 * system to favor processes which haven't run much
			 * recently, and to round-robin among other
			 * processes. NOTE: The scheduling alteration is a
			 * non sequitur to kernel running lwp's. In fact, if
			 * p_pri changes while the process is still on the
			 * run queue, a panic from remrq may result
			 */
#ifdef LWP
			if (!runthreads)
#endif	/* LWP */
			{
				/*
				 * It is possible that this process has been
				 * placed back on the run queue and that the
				 * CPU has not changed its uunix pointer, so
				 * do the right thing ...
				 */
				int	wasonq = onrq(p);
				p->p_cpticks++;
				if (++p->p_cpu == 0)
					p->p_cpu--;
				if ((p->p_cpu & 3) == 0) {
					(void) setpri(p);
					if (p->p_pri >= PUSER) {
						if (wasonq)
							remrq(p);
						p->p_pri = p->p_usrpri;
						if (wasonq)
							setrq(p);
					}
				}
			}
		}
	}
}

#endif /* MULTIPROCESSOR */

int	dk_ndrive = DK_NDRIVE;
/*
 * Gather statistics on resource utilization.
 *
 * We make a gross assumption: that the system has been in the
 * state it is in (user state, kernel state, interrupt state,
 * or idle state) for the entire last time interval, and
 * update statistics accordingly.
 *
 * In the MP version, only one processor per hardclock interrupt (level
 * 10) calls this routine.  Therefore, one
 * processor collects CPU time data for all of the processors present.
 * 
 */
/*ARGSUSED*/
#ifdef MULTIPROCESSOR

gatherstats(pc, ps)
	caddr_t         pc;
	int             ps;
{
	extern int procset;
	register int	cix, cs;
	register struct user *up;
	register struct proc *pp;

	/*
	 * Gather stats for each processor.
	 */
	for (cix=0; cix < nmod; ++cix)
		if ((procset & (1<<cix)) &&
		    (PerCPU[cix].cpu_enable) &&
		    (PerCPU[cix].cpu_exists)) {

			if (PerCPU[cix].cpu_idle)
				cs = CP_IDLE;
			else if (PerCPU[cix].cpu_supv)
				cs = CP_SYS;
			else if ((PerCPU[cix].noproc) ||
				 ((up = PerCPU[cix].uunix) == NULL) ||
				 (up == &idleuarea) ||
				 ((pp = PerCPU[cix].masterprocp) == NULL) ||
				 (pp == &idleproc))
				cs = CP_IDLE;
			else if (pp->p_nice > PZERO)
				cs = CP_NICE;
			else
				cs = CP_USER;

			mp_time[cix][cs]++;
			cp_time[cs]++;

#ifdef	XP_SPIN
			if ((cs == CP_SYS) &&
			    (PerCPU[cix].cpu_supv == 2))
				xp_time[cix][XP_SPIN] ++;
#endif	XP_SPIN

#ifdef	XP_DISK
			if ((cs == CP_IDLE) &&
			    (dk_busy != 0))
				xp_time[cix][XP_DISK] ++;
#endif	XP_DISK

#ifdef	XP_SERV
			if ((cs == CP_SYS) &&
			    (PerCPU[cix].cpu_supv == 3))
				xp_time[cix][XP_SERV] ++;
#endif	XP_SERV

		}

	/*
	 * Collect disk busy data.
	 */
	for (cix = 0; cix < dk_ndrive; cix++)
		if (dk_busy & (1 << cix))
			dk_time[cix]++;
}

#else /* !MULTIPROCESSOR */

gatherstats(pc, ps)
	caddr_t pc;
	int ps;
{
	register int cpstate, s;

	/*
	 * Determine what state the cpu is in.
	 */
	if (USERMODE(ps)) {
		/*
		 * CPU was in user state.
		 */
		if (u.u_procp->p_nice > NZERO)
			cpstate = CP_NICE;
		else
			cpstate = CP_USER;
	} else {
		/*
		 * CPU was in system state.  If profiling kernel
		 * increment a counter.  If no process is running
		 * then this is a system tick if we were running
		 * at a non-zero IPL (in a driver).  If a process is running,
		 * then we charge it with system time even if we were
		 * at a non-zero IPL, since the system often runs
		 * this way during processing of system calls.
		 * This is approximate, but the lack of true interval
		 * timers makes doing anything else difficult.
		 */
		cpstate = CP_SYS;
		if (noproc && BASEPRI(ps))
			cpstate = CP_IDLE;
#ifdef GPROF
#ifdef sun
		/*
		 * For Sun, this stuff is done in sun/kprof.s
		 */
#else
		s = pc - s_lowpc;
		if (profiling < 2 && s < s_textsize)
			kcount[s / (HISTFRACTION * sizeof (*kcount))]++;
#endif
#endif
	}
	/*
	 * We maintain statistics shown by user-level statistics
	 * programs:  the amount of time in each cpu state, and
	 * the amount of time each of dk_ndrive ``drives'' is busy.
	 */
	cp_time[cpstate]++;
	for (s = 0; s < dk_ndrive; s++)
		if (dk_busy & (1 << s))
			dk_time[s]++;
}
#endif MULTIPROCESSOR

#ifdef	MULTIPROCESSOR
static
do_SOWEUPC()
{
	register struct user *up;
	register struct proc *p;
	if ((up = uunix) && (p = up->u_procp)) {
		p->p_flag |= SOWEUPC;
		aston();
	}
}
#endif	MULTIPROCESSOR

/*
 * Software priority level clock interrupt.
 * Run periodic events from timeout queue.
 */
softclock(ps)
	int ps;
{
#ifdef	MULTIPROCESSOR
	int cix;
	struct user *up;
#endif	MULTIPROCESSOR

#ifdef MEASURE_KERNELSTACK
/*
 * Print out stack statistics, if necessary
 */
	if (print_minkernstack) {
		register int s;
		register int sp;

		s = splclock();
		print_minkernstack = 0;
		sp = minkernstack;
		(void) splx(s);
		printf("minkernstack = 0x%x\n", sp);
	}

	if (print_minintrstack) {
		register int s;
		register int sp;

		s = splclock();
		print_minintrstack = 0;
		sp = minintrstack;
		(void) splx(s);
		printf("minintrstack = 0x%x\n", sp);
	}
#endif MEASURE_KERNELSTACK

	for (;;) {
		register struct callout *p1;
		register caddr_t arg;
		register int (*func)();
		register int a, s;

		s = splclock();
		if (((p1 = calltodo.c_next) == 0) || (p1->c_time > 0)) {
			(void) splx(s);
			break;
		}
		arg = p1->c_arg; func = p1->c_func; a = p1->c_time;
		calltodo.c_next = p1->c_next;
		p1->c_next = callfree;
		callfree = p1;
		(void) splx(s);
		(*func)(arg, a);
	}

	/*
	 * If trapped user-mode and profiling, give it
	 * a profiling tick.
	 */
	if (USERMODE(ps)) {
		register struct proc *p = u.u_procp;

		if (u.u_prof.pr_scale) {
			p->p_flag |= SOWEUPC;
			aston();
		}
	}

#ifdef	MULTIPROCESSOR
	/*
	 * If any other cpu is profiling, give it a profiling
	 * tick if it is in user mode.
	 */
	for (cix = 0; cix < nmod; cix++) {
		if ((cix != cpuid) &&
		    (procset & (1<<cix)) &&
		    PerCPU[cix].cpu_exists &&
		    !PerCPU[cix].noproc &&
		    (up = PerCPU[cix].uunix) &&
		    (up != &idleuarea) &&
		    up->u_prof.pr_scale)
			xc_oncpu(0, 0, 0, 0,
				cix, do_SOWEUPC);
	}
#endif	MULTIPROCESSOR
}

/*
 * Arrange that (*fun)(arg) is called in t/hz seconds.
 */
timeout(fun, arg, t)
	int (*fun)();
	caddr_t arg;
	register int t;
{
	register struct callout *p1, *p2, *pnew;
	register int s = splclock();

	if (t <= 0)
		t = 1;
	pnew = callfree;
	if (pnew == NULL)
		panic("timeout table overflow");
	callfree = pnew->c_next;
	pnew->c_arg = arg;
	pnew->c_func = fun;
	for (p1 = &calltodo; (p2 = p1->c_next) && p2->c_time < t; p1 = p2)
		if (p2->c_time > 0)
			t -= p2->c_time;
	p1->c_next = pnew;
	pnew->c_next = p2;
	pnew->c_time = t;
	if (p2)
		p2->c_time -= t;
	(void) splx(s);
}

/*
 * untimeout is called to remove a function timeout call
 * from the callout structure.
 */
untimeout(fun, arg)
	int (*fun)();
	caddr_t arg;
{
	register struct callout *p1, *p2;
	register int s;

	s = splclock();
	for (p1 = &calltodo; (p2 = p1->c_next) != 0; p1 = p2) {
		if (p2->c_func == fun && p2->c_arg == arg) {
			if (p2->c_next && p2->c_time > 0)
				p2->c_next->c_time += p2->c_time;
			p1->c_next = p2->c_next;
			p2->c_next = callfree;
			callfree = p2;
			break;
		}
	}
	(void) splx(s);
}

/*
 * MICROFUDGE is used by "hzto" to relax its calculation
 * just a little bit; if an integer number of ticks take
 * us to less than MICROFUDGE microseconds before the
 * destination time, we call it "enough" and do not wait
 * the extra tick.
 */
#ifndef MICROFUDGE
#define	MICROFUDGE	10
#endif

/*
 * Compute number of hz until specified time.
 * Used to compute third argument to timeout() from an
 * absolute time.
 */
hzto(tv)
	struct timeval *tv;
{
	register long ticks, sec, usec;
	int s = splclock();

	/*
	 * Compute number of ticks we will see between now and
	 * the target time; returns "1" if the destination time
	 * is before the next tick, so we always get some delay,
	 * and returns 0x7fffffff ticks if we would overflow.
	 */
	sec = tv->tv_sec - time.tv_sec;
	usec = tv->tv_usec - time.tv_usec;
	if (usec < 0) {
		sec--;
		usec += 1000000;
	}
	ticks = usec * hz / 1000000;	/* integer ticks in usec delay */
	usec -= (ticks * 1000000) / hz;	/* ticks remaining */

	if (usec > MICROFUDGE)
		ticks++;
	/*
	 * Compute ticks, accounting for negative and overflow as above.
	 * Overflow protection kicks in at about 70 weeks for hz=50
	 * and at about 35 weeks for hz=100.
	 */
	if ((sec < 0) || ((sec == 0) && (ticks < 1)))
		ticks = 1;			/* protect vs nonpositive */
	else if (sec > (0x7fffffff - ticks) / hz)
		ticks = 0x7fffffff;		/* protect vs overflow */
	else
		ticks += sec * hz;		/* common case */

	(void) splx(s);
	return (ticks);
}

profil()
{
	register struct a {
		short	*bufbase;
		unsigned bufsize;
		unsigned pcoffset;
		unsigned pcscale;
	} *uap = (struct a *)u.u_ap;
	register struct uprof *upp = &u.u_prof;

	/*
	 * The low-order bit of the scale is irrelevant anyway, and a scale of
	 * 1 is supposed to be equivalent to a scale of 0 (both should turn
	 * profiling off), so we clear the low-order bit.
	 */
	upp->pr_base = uap->bufbase;
	upp->pr_size = uap->bufsize;
	upp->pr_off = uap->pcoffset;
	upp->pr_scale = uap->pcscale & ~01;
}

/*
 * If MICRO is defined, we use the fast routine microtime() in the NTP
 * distribution; if neither it nor an external clock has been
 * configured, we compile the following code.
 */
#if !defined(MICRO) && !defined(EXT_CLOCK)
#if defined(SUN4_330) || defined(SUN4_470) || defined(sun4c) || defined(sun4m)
#define	HI_RES_CLOCK
#endif

uniqtime(tv)
	register struct timeval *tv;
{
	static struct timeval last;		/* last value returned */
	static struct timeval now;		/* current time value */
#ifdef	HI_RES_CLOCK
	register int cr;			/* nsec counter register */
	register int ib;			/* usec past last tick */
#endif
	int s;

	s = splclock();	/* protect accesses to now, last, time */
	now = time;
#ifdef	HI_RES_CLOCK
	/*
	 * Some models have a high resolution timer that we can use to
	 * figure out the microsecond-precision time.
	 *
	 * If the "limit reached" bit appears in the value register and is
	 * turned on, we have a pending interrupt; add a tick's worth of time
	 * to the clock.
	 *
	 * If the "limit reached" bit is not reflected in the counter value
	 * register, then this may be slow by 1/100 sec if there is a pending
	 * clock interrupt; reading the bit from the limit register would be a
	 * bad thing.
	 *
	 * The limit bit may turn on AS WE REACH the limit. Be ready
	 * to handle both cases here.
	 *
	 * XXX - it would be nice to be able to have this code figure
	 * out if the clock is supported and if it is there without
	 * looking at "cpu".
	 */
#if !defined(sun4c) && !defined(sun4m)
	switch (cpu) {
#ifdef SUN4_330
	case CPU_SUN4_330:
#endif
#ifdef SUN4_470
	case CPU_SUN4_470:
#endif
#endif	/* !sun4c && !sun4m */
		cr = COUNTER->counter10;
		ib = (cr & CTR_USEC_MASK) >> CTR_USEC_SHIFT;
		if (cr & CTR_LIMIT_BIT)
			ib = (ib % tick) + tick;
		BUMPTIME(&now, ib);
#if !defined(sun4c) && !defined(sun4m)
		break;
	default:	/* kernel supports, cpu does not */
		break;
	}
#endif	/* !sun4c && !sun4m */
#endif /* HI_RES_CLOCK */

	/*
	 * Try to keep the return value of uniqtime() monotonicly
	 * increasing in the face of repeated calls, but don't be
	 * obsessive about it in the face of large differences.
	 */
	if ((now.tv_sec > last.tv_sec) ||	/* higher second, or */
	    ((now.tv_sec == last.tv_sec) &&	/* same second and */
	    (now.tv_usec > last.tv_usec)) ||	/* higher microsecond, or */
	    ((last.tv_sec - now.tv_sec) > 5)) {	/* definitely back in time */
		last = now;
	} else {
		BUMPTIME(&last, 1);	/* keep it monotonicly ascending */
	}

	*tv = last;
	(void) splx(s);
}
#endif /* MICRO */

